@@ -232,6 +232,10 @@ regidx_t *regidx_init(const char *fname, regidx_parse_f parser, regidx_free_f fr
                 parser = regidx_parse_bed;
             else if ( len>=4 && !strcasecmp(".bed",fname+len-4) )
                 parser = regidx_parse_bed;
+            else if ( len>=4 && !strcasecmp(".vcf",fname+len-4) )
+                parser = regidx_parse_vcf;
+            else if ( len>=7 && !strcasecmp(".vcf.gz",fname+len-7) )
+                parser = regidx_parse_vcf;
             else
                 parser = regidx_parse_tab;
         }
@@ -258,7 +262,11 @@ regidx_t *regidx_init(const char *fname, regidx_parse_f parser, regidx_free_f fr
     }
 
     free(str.s);
-    hts_close(fp);
+    if ( hts_close(fp)!=0 )
+    {
+        fprintf(stderr,"[%s] Error: close failed .. %s\n", __func__,fname);
+        goto error;
+    }
     return idx;
 
 error:
@@ -388,12 +396,11 @@ int regidx_overlap(regidx_t *regidx, const char *chr, uint32_t beg, uint32_t end
         {
             int iend = iBIN(end);
             if ( iend > list->nidx ) iend = list->nidx;
-            for (i=ibeg; i<iend; i++)
+            for (i=ibeg; i<=iend; i++)
                 if ( list->idx[i] ) break;
-            if ( i==iend ) return 0;
+            if ( i>iend ) return 0;
             i = list->idx[i];
         }
-
         for (ireg=i-1; ireg<list->nreg; ireg++)
         {
             if ( list->reg[ireg].beg > end ) return 0;   // no match, past the query region
@@ -488,13 +495,20 @@ int regidx_parse_tab(const char *line, char **chr_beg, char **chr_end, uint32_t
     {
         ss = se+1;
         *end = strtod(ss, &se);
-        if ( ss==se ) *end = *beg;
+        if ( ss==se || (*se && !isspace(*se)) ) *end = *beg;
         else if ( *end==0 ) { fprintf(stderr,"Could not parse tab line, expected 1-based coordinate: %s\n", line); return -2; }
         else (*end)--;
     }
     return 0;
 }
 
+int regidx_parse_vcf(const char *line, char **chr_beg, char **chr_end, uint32_t *beg, uint32_t *end, void *payload, void *usr)
+{
+    int ret = regidx_parse_tab(line, chr_beg, chr_end, beg, end, payload, usr);
+    if ( !ret ) *end = *beg;
+    return ret;
+}
+
 int regidx_parse_reg(const char *line, char **chr_beg, char **chr_end, uint32_t *beg, uint32_t *end, void *payload, void *usr)
 {
     char *ss = (char*) line;
