@@ -25,6 +25,7 @@ DEALINGS IN THE SOFTWARE.  */
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 #include <htslib/hts.h>
 #include "bcftools.h"
 #include "version.h"
@@ -44,6 +45,22 @@ void error(const char *format, ...)
     exit(-1);
 }
 
+void error_errno(const char *format, ...)
+{
+    va_list ap;
+    int e = errno;
+    va_start(ap, format);
+    vfprintf(stderr, format, ap);
+    va_end(ap);
+    if (e) {
+        fprintf(stderr, ": %s\n", strerror(e));
+    } else {
+        fprintf(stderr, "\n");
+    }
+    exit(-1);
+}
+
+
 const char *hts_bcf_wmode(int file_type)
 {
     if ( file_type == FT_BCF ) return "wbu";    // uncompressed BCF
